/* Sources:
**   http://www.nongnu.org/avr-libc/user-manual/modules.html
** FUSE Calculator:
**   http://www.engbedded.com/cgi-bin/fc.cgi/
**
** FUSEs:  H:0xC8  L:0xEF
**   avrdude -c usbasp -p atmega32 -u -U hfuse:w:0xC8:m -U lfuse:w:0xEF:m
** Para leer los FUSEs:
**   avrdude -c usbasp -p atmega32 -U hfuse:r:hfuse.txt:m -U lfuse:r:lfuse.txt:m
**
** Grabar programa con una de las siguiente lineas:
**   avrdude -c usbasp -p atmega32 -U flash:w:Firmware.elf.hex
**   avrdude -c usbasp -p atmega32 -U flash:w:Firmware.elf.hex -U eeprom:w:Firmware.elf.eep.hex
*/

#define F_CPU 16000000UL

#include <avr/io.h>
#include <avr/interrupt.h>      // required by usbdrv.h
#include <avr/pgmspace.h>   	// required by usbdrv.h
#include <avr/wdt.h>            // WatchDog
#include <util/delay.h>			// _delay_ms()
#include <avr/eeprom.h>
#include <string.h>

#define __FIRMWARE__
#include "defs.h"
#include "usbdrv/usbconfig.h"
#include "usbdrv/usbdrv.h"		// V-USB driver
//#include "usbdrv/oddebug.h"	// V-USB driver DEBUG (no lo usamos)

uchar EEMEM CosasEnLaEeprom[]="Ejemplo de contenido guardado en la EEPROM";
uchar EEMEM MasCosasEnLaEeprom[]={1,2,3,4,5,6,9,8,7,6,5,4,3,2,1};

typedef union word{
    unsigned    word;
    uchar       byte[2];
}word_t;

//==========================================================================
//=========================== Variables and Function Declarations ==========
//==========================================================================

// Variables del buffer en RAM
#define BUFFER_SIZE 500
static uchar Buffer[BUFFER_SIZE]={1}; //inicializar a todos 1
static uchar reportBuffer[4]; // como maximo 8 bytes

// Variables de control de usbFunctionWrite/Read:
static uchar 	currentAddress;    // es el offset usado para gravar sobre la eeprom.
static word_t 	bytesRemaining;    // contador de decremento de cantidad de bytes restantes
static uchar	transferType;      // identificador del tipo de transferencia
static uchar   *nextTransferByte; // puntero que se incrementa para recorrer el buffer

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};

static uchar reportBuffer[4];

static void softwareInit(void);
static void hardwareInit(void);

//==========================================================================
//=========================== main =========================================
//==========================================================================

int main(void)
{
    uchar i;

    wdt_enable(WDTO_1S);

    hardwareInit();
	softwareInit();
    usbInit();

    usbDeviceDisconnect();
    i = 0;
    while(--i){     // Falsa desconexion de USB por menos de 250 ms
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();			// Activar interrupciones globales

    while(1)
    {
		wdt_reset();
		usbPoll();
    }

    return 0;
}


//==========================================================================
//=========================== Inicializacion ===============================
//==========================================================================

static void softwareInit(void)
{
	// Inicializar memorias y buffers
}

static void hardwareInit(void)
{
    // Inicializar puertos y modulos internos
    LED_PORT_DDR = 0xFF;
    LED_PORT_WRITE = 0x0F;
	DDRD = ~(USB_MASK);
	PORTD = ~(USB_MASK);

}


//==========================================================================
//=========================== usbdrv =======================================
//==========================================================================

usbMsgLen_t usbFunctionSetup(uchar setupData[8])
{
    usbRequest_t  *rq = (void *)setupData;
    usbMsgLen_t len;
    //
    // En este ejemplo introducimos los tipos de peticiones segun como se identifique el dispositivo.
    // El tipo de peticion se define segun mayormente 3 categorias, standard, class y vendor.
    //
    // Las peticiones Standard son comunes para todos los dispositivos USB.
    // Las peticiones Class o de clases son comunes solo dentro de un conjunto de clases de drivers.
    // Por ejemplo, todos los dispositivos de clase HID tendran un conjunto comun de
    //       peticiones especificas de esa clase.
    // Estas seran diferentes de los dispositivos de clases de comunicaciones, y diferentes de nuevo
    //       de los de almacenamiento.
    // Por ultimo estan las peticione definidas por el fabricante (Vendor defined requests).
    // Estas son peticiones que tu, como diseñador USB, puede asignar y son normalmente diferentes
    //       de dispositivo a dispositivo, pero solo depende de tu imaginacion.
    //

    switch( rq->bmRequestType & USBRQ_TYPE_MASK ){
        case USBRQ_TYPE_VENDOR:
            //
            // Una peticion de tipo Vendor permite al desarrollador definir su propio
            //    set de transmiciones, esto significa que el campo bRequest esta
            //    completamente disponible para el programador.
            // La desventaja que esto posee es que en windows se requiere de un driver
            //    para poder utilizar dichas peticiones.
            //
            // Lo siguiente debe ser compatible con los softwares de los ejemplos anteriores.
            switch(rq->bRequest){
                case RQ_CUSTOM___ECHO:
                    reportBuffer[0] = rq->wValue.bytes[0];
                    reportBuffer[1] = rq->wValue.bytes[1];
                    reportBuffer[2] = rq->wIndex.bytes[0];
                    reportBuffer[3] = rq->wIndex.bytes[1];
                    usbMsgPtr = reportBuffer;
                    return 4;
                case RQ_CUSTOM___SET_LEDS_STATUS:
                    LED_PORT_WRITE = rq->wValue.bytes[0];
                    return 0;
                case RQ_CUSTOM___GET_LEDS_STATUS:
                    reportBuffer[0] = LED_PORT_READ;
                    usbMsgPtr = reportBuffer;
                    return 1;
            }
            break;

        case USBRQ_TYPE_CLASS:
            //
            // Las peticiones de tipo Class estan enumeradas en el estandard dependiendo
            //    de la clase del dispositivo.
            // La clase que se utiliza en estos ejemplos es la HID o Human Interface Device que se
            //     indica en el descriptor.
            // Las peticiones que se utilizan para proposito general son las 2 aqui abajo enumeradas.
            //    USBRQ_HID_GET_REPORT - Para una transaccion de Disp. a Host
            //    USBRQ_HID_SET_REPORT - Para una transaccion de Host a Disp.
            //
            switch(rq->bRequest){
                case USBRQ_HID_GET_REPORT:
                    //
                    // Por estandard, el parametro wValue recibido contiene las siguientes asignaciones:
                    //    wValue: ReportType (highbyte), ReportID (lowbyte)
                    // Lo cual no necesitamos respetar.
                    // En nuestro ejemplo usaremos el lowbyte de wIndex como identificador del reporte.
                    // Un reporte en este caso es solo una transferencia de datos.
                    //
                    switch( rq->wIndex.bytes[0] ){
                        case USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO:
							transferType = USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO;
							// wValue es el offset dentro del buffer.
							// Usado para poder transmitir buffer mas grandes que 255
							nextTransferByte = &Buffer[rq->wValue.word];
                            bytesRemaining.word = BUFFER_SIZE;
                            if(bytesRemaining.word > rq->wLength.word)
                                bytesRemaining.word = rq->wLength.word;
                            return USB_NO_MSG;
                            //
                            // Retornando USB_NO_MSG desde USBRQ_HID_GET_REPORT informamos al driver que
                            //    se desea utilizar la funcion usbFunctionRead() para enviar los datos al Host.
                            // Esta funcion debe usarse siempre que los datos se deban generar en el momento.
                            // Para enviar datos almacenados en memoria se usa el mismo metodo visto anteriormente.
                            // Cargamos la direccion de usbMsgPtr y retornamos un valor de 8 bits menor o igual a 254
                            // Si activamos la opcion para transferencias largas, podemos
                            //    realizar transferencias de 16bits. USB_CFG_LONG_TRANSFERS
                            //

                        case USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ:
							// Esto solo funcionara si se encuentra activada USB_CFG_LONG_TRANSFERS
							//#define USB_CFG_LONG_TRANSFERS          1
							// Recuerde que activar muchas opciones hace que el driver ocupe mas espacio de flash.
                            len = BUFFER_SIZE;
                            if(len > rq->wLength.word)
                                len = rq->wLength.word;
                            usbMsgPtr = &Buffer[rq->wValue.word];
							return len;

                        case USB_HID_CUSTOM_RQ___BUFFER_EEPROM:
							transferType = USB_HID_CUSTOM_RQ___BUFFER_EEPROM;
		            		bytesRemaining.byte[0] = rq->wLength.bytes[0];
							bytesRemaining.byte[1] = 0;
		            		currentAddress = 0;
                            return USB_NO_MSG;


						case USB_HID_CUSTOM_RQ___INFO:
                            //
                            // A veces es necesario que nuestro dispositivo identifique sus capacidades,
                            //    como el tamaño del buffer, los modos de operacion, etc.
                            //
							// Un unmero de 16 bits
							reportBuffer[0] = (( 2560 & 0xFF00 ) >> 8);  // High Part
							reportBuffer[1] = ( 2560 & 0x00FF ); 		 // Low Part
							// BUFFER SIZE
							reportBuffer[2] = (( BUFFER_SIZE & 0xFF00 ) >> 8);  // High Part
							reportBuffer[3] = ( BUFFER_SIZE & 0x00FF ); 		// Low Part
							usbMsgPtr = reportBuffer;
							return 4;

						case USB_HID_CUSTOM_RQ___MODES:
							switch( rq->wIndex.bytes[1] )
								{
								case 1:
									reportBuffer[0] = 10;
									reportBuffer[1] = 125;
									reportBuffer[2] = 14.75*4;
									break;
								case 2:
									reportBuffer[0] = 8;
									reportBuffer[1] = 1;
									reportBuffer[2] = 17.5*4;
									break;
								case 3:
									reportBuffer[0] = 8;
									reportBuffer[1] = 125;
									reportBuffer[2] = 15*4;
									break;
								default:
									reportBuffer[0] = 0;
									reportBuffer[1] = 0;
									reportBuffer[2] = 17.5*4;
									break;
								}
							usbMsgPtr = reportBuffer;
							return 3;

						case USB_HID_CUSTOM_RQ___LEDS_STATUS:
                            reportBuffer[0]=LED_PORT_READ;
                            usbMsgPtr = reportBuffer;
							return 1;
                    }
                    break;

                case USBRQ_HID_SET_REPORT:
                    switch( rq->wIndex.bytes[0] ){
                        case USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ:
							// Esta es una operacion ilegal.
							// El driver no puede escribir directamente sobre la memoria.
							// El driver posee su buffer interno de solo 8 bytes para datos.
							// Recibir un buffer completo requiriria de una gran cantidad de memoria.
                            // Solo puede usar el metodo de usbFunctionWrite()
                            LED_PORT_WRITE = 0xFF;

                        case USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO:
							transferType = USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO;
							nextTransferByte = &Buffer[rq->wValue.word];
                            bytesRemaining.word = BUFFER_SIZE;
                            if(bytesRemaining.word > rq->wLength.word)
                                bytesRemaining.word = rq->wLength.word;
                            return USB_NO_MSG;
                            //
                            // Retornando USB_NO_MSG desde USBRQ_HID_GET_REPORT informamos al driver que
                            //    se desea utilizar la funcion usbFunctionWrite() para recibir datos desde el host.
                            // Esta funcion debe usarse siempre que los datos deban ser generados en
                            //    el momento de transferirse, como el resultado de un ADC.
                            // Si los datos residen en memoria puede transmitirse directamente como ya se explico.
                            //
                        case USB_HID_CUSTOM_RQ___BUFFER_EEPROM:
							transferType = USB_HID_CUSTOM_RQ___BUFFER_EEPROM;
		            		bytesRemaining.byte[0] = rq->wLength.bytes[0];
							bytesRemaining.byte[1] = 0;
		            		currentAddress = 0;
                            return USB_NO_MSG;

						case USB_HID_CUSTOM_RQ___LEDS_STATUS:
							switch( rq->wIndex.bytes[1] ){
                                case 1:
                                    LED_PORT_WRITE = 0x0f;
                                    break;
                                case 2:
                                    LED_PORT_WRITE = 0x55;
                                    break;
                                case 3:
                                    LED_PORT_WRITE = 0x01;
                                    break;
                                case 4:
                                    LED_PORT_WRITE = rq->wValue.bytes[0];
                                    break;
                                default:
                                    break;
								}
                            return 0;
                    }
                    break;

            }
            break;
        case USBRQ_TYPE_STANDARD:
            //
            // Este ultimo tipo de peticion no lo usaremos en estos ejemplos.
            // Ya que es solo necesario si nuestra intencion es desarrollar un dispositivo estandard.
            //
            break;

    }
    return 0;

}

//
// Las siguientes funciones son de especial importancia cuando debes calcular
//    u obtener un dato antes de transmitirlo, pero no tanto cuando todos los
//    datos se encuentran en memoria.
// En este ejemplo se utiliza un buffer en memoria con ambos sistemas, pero
//    recuerda que el metodo mas eficiente es transmitir directo desde la memoria
//    en la memoria en la funcion usbFunctionSetup.
// Es decir, el metodo correcto es USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ.
// Lo siguiente solo va a modo de demostracion.
//
//==========================================================================
//============ usbFunctionRead - Enviar datos al host ======================
//==========================================================================
//#define USB_CFG_IMPLEMENT_FN_READ       1
uchar   usbFunctionRead(uchar *data, uchar len)
{                                      // "len" es como maximo 8 bytes
	if(bytesRemaining.word == 0)
		return 0;

    if(len > bytesRemaining.word) // si quedan menos de 8 bytes
        len = bytesRemaining.byte[0];

	switch(transferType){
		case USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO:
			// Calcula tus datos aqui y cargalos en el vector data
			memcpy( data , nextTransferByte , len );
			nextTransferByte += len;
			break;
		case USB_HID_CUSTOM_RQ___BUFFER_EEPROM:
            // Calcula tus datos aqui y cargalos en el vector data
		    eeprom_read_block(data, (uchar *)0 + currentAddress, len);
    		currentAddress += len;
			break;
	}

    bytesRemaining.word -= len;
    return len; // se retorna la cantidad de bytes transmitidos
    // si len es menor que 8, entonces esta es la ultima transmicion.
}

//==========================================================================
//============ usbFunctionWrite - Recibir datos desde el host ==============
//==========================================================================
//#define USB_CFG_IMPLEMENT_FN_WRITE      1
uchar   usbFunctionWrite(uchar *data, uchar len)
{
	if(bytesRemaining.word == 0)
		return 1;

    if(len > bytesRemaining.word)
        len = bytesRemaining.byte[0];

	switch(transferType){
		case USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO:
			memcpy(nextTransferByte, data, len);
			nextTransferByte += len;
			break;
		case USB_HID_CUSTOM_RQ___BUFFER_EEPROM:
		    eeprom_write_block(data, (uchar *)0 + currentAddress, len);
    		currentAddress += len;
			break;
	}

    bytesRemaining.word -= len;
    return (bytesRemaining.word == 0);
    // retorna 1 si esta es el ultimo paquete.
}



