/*
** libusb API:
**    http://libusb.sourceforge.net/api-1.0/
**    http://libusb.sourceforge.net/api-1.0/group__syncio.html
**    http://libusb.sourceforge.net/api-1.0/group__dev.html#g7df95821d20d27b5597f1d783749d6a4
**
*/

#include <stdio.h>
#include <string.h>
#include "../Firmware/usbdrv/usbconfig.h"

#include "../Firmware/defs.h"

#include <usb.h>

#define USBRQ_HID_GET_REPORT    0x01
#define USBRQ_HID_SET_REPORT    0x09
// Note que aqui cambia de USB_TYPE_VENDOR a USB_TYPE_CLASS
#define CTRL_IN  (USB_TYPE_CLASS | USB_RECIP_DEVICE | USB_ENDPOINT_IN)
#define CTRL_OUT (USB_TYPE_CLASS | USB_RECIP_DEVICE | USB_ENDPOINT_OUT)


// Note aqui que agregamos el numero de serie
usb_dev_handle* usb_find_and_open(int vendorID, char *vendor, int productID, char *product, char *serial);
// int usb_control_msg( device_handler, request_type, request, wValue, wIndex, data, wLength, timeout )


int main()
{
    const unsigned char rawVid[2] = {USB_CFG_VENDOR_ID};
    const unsigned char rawPid[2] = {USB_CFG_DEVICE_ID};
    char vendor[] = {USB_CFG_VENDOR_NAME, 0};
    char product[] = {USB_CFG_DEVICE_NAME, 0};
    char serie[] = {USB_CFG_SERIAL_NUMBER, 0};
    int vendorID, productID;
    usb_dev_handle* usbdev;
    char buffer[500]={0};
    int rc;
    char Buffer2[500]={0x66};
    char Buffer3[500]="Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.";
    unsigned int max_buffer_size;
    int wLength, wValue, wIndex;
    int i;

    vendorID = rawVid[1] * 256 + rawVid[0];
    productID = rawPid[1] * 256 + rawPid[0];

    printf("Buscando: VID=%x, PID=%x, vendor=%s, product=%s\n",vendorID,productID,vendor,product);

	usb_init();

    //
    // Implementamos una funcion que nos permita buscar el dipsositivo entre todos los conectados.
    // En este caso incluimos el numero de serie.
    //
    usbdev = usb_find_and_open(vendorID, vendor, productID, product, serie);
    if (usbdev == NULL)
    {
	    fprintf(stderr, "failed to find device\n");
        return 0;
    }

    printf("Encontrado.\n");

    //=======================================================================
    //=======================================================================

    wLength = 0;
    wIndex = (2 <<8) | USB_HID_CUSTOM_RQ___LEDS_STATUS;
    wValue = 0;
    rc = usb_control_msg( usbdev, CTRL_OUT, USBRQ_HID_SET_REPORT, wValue, wIndex, 0, wLength, 1000 );
    if ( rc < 0){
        fprintf(stderr, "failed to transfer a few bytes\n");
        usb_close(usbdev);
        return 0;
    }
    printf("Realizado USB_HID_CUSTOM_RQ___LEDS_STATUS modo 2 - correctamente.\n");

    //=======================================================================
    //=======================================================================

    //
    // Leemos informacion del dispositivo, como el largo del buffer interno.
    //
    wLength = 8;
    wIndex = USB_HID_CUSTOM_RQ___INFO;
    wValue = 0;
    rc = usb_control_msg( usbdev, CTRL_IN, USBRQ_HID_GET_REPORT, wValue, wIndex, buffer, wLength, 1000 );
    if ( rc < 0){
	    fprintf(stderr, "failed to transfer a few bytes\n");
        usb_close(usbdev);
        return 0;
    }
    if ( rc!=4 ){
	    fprintf(stderr, "failed to transfer safelly a few bytes\n");
        usb_close(usbdev);
        return 0;
    }
    max_buffer_size = ((unsigned char)buffer[2]<<8) | (unsigned char)buffer[3];
    printf("Realizado USB_HID_CUSTOM_RQ___INFO correctamente. Buffer interno: %d\n", max_buffer_size);
printf("----\n");

    //=======================================================================
    //=======================================================================

    //
    // Transmitimos un buffer de 500 elementos.
    // El maximo permitido por transmicion es de 254 elementos. (contador de 8 bits)
    // Pero si se encuentra activadas las transferencias largas, hasta 65k son posibles
    // El metodo que se usa es el de generacion al envio.
    //

    wLength = sizeof(Buffer2);
    if (wLength > max_buffer_size )
        wLength = max_buffer_size;

    printf("Transmitiendo buffer de %d elementos.\n", wLength);
    wIndex = USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO;
    wValue = 0;
    rc = usb_control_msg( usbdev, CTRL_OUT, USBRQ_HID_SET_REPORT, wValue, wIndex, Buffer2, max_buffer_size, 1000 );
    if ( rc < 0){
        fprintf(stderr, "failed to transfer a few bytes\n");
        usb_close(usbdev);
        return 0;
    }
    printf("Realizado USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO correctamente.\n");


    //
    // Recibimos de nuevo el buffer de 500 elementos.
    // Verificamos que sea lo mismo que lo enviado.
    //
    wLength = sizeof(buffer);
    wIndex = USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO;
    wValue = 0;
    rc = usb_control_msg( usbdev, CTRL_IN, USBRQ_HID_GET_REPORT, wValue, wIndex, buffer, wLength, 1000 );
    if ( rc <= 0){
        fprintf(stderr, "failed to transfer a few bytes. Error: %d\n", rc);
        usb_close(usbdev);
        return 0;
    }
    printf("Recibido buffer de %d elementos.\n", rc);
    for (i = 0 ; i < rc ; i++)
        {
        if ( buffer[i] != Buffer2[i] ){
            printf("ERROR realizando USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO - no se recibio el mismo valor enviado.\n");
            printf("\t%d != %d.\n",buffer[i] ,Buffer2[i]);
            break;
            }
        }
    printf("Terminado USB_HID_CUSTOM_RQ___BUFFER_PRIMARIO.\n");
printf("----\n");
    //=======================================================================
    //=======================================================================

    //
    // Transmitimos un buffer de 500 elementos.
    // El maximo permitido por transmicion es de 16 bits de tamaño.
    // El metodo que se usa es el de envio de una vez.
    // NOTE: que debido a la forma de implementar en el firmware, esta operacion es ilegal.
    //     ya que no se puede transmitir directo hacia la memoria sin usar la
    //     funcion usbFunctionWrite(). Pero implementado correctamente, esto
    //     sera transparente para el software.
    //

    wLength = sizeof(Buffer3);
    if (wLength > max_buffer_size )
        wLength = max_buffer_size;

    printf("Transmitiendo buffer de %d elementos.\n", wLength);
    wIndex = USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ;
    wValue = 0;
    rc = usb_control_msg( usbdev, CTRL_OUT, USBRQ_HID_SET_REPORT, wValue, wIndex, Buffer3, max_buffer_size, 1000 );
    if ( rc < 0){
        fprintf(stderr, "failed to transfer a few bytes\n");
        usb_close(usbdev);
        return 0;
    }
    printf("Realizado USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ correctamente.\n");


    //
    // Recibimos de nuevo el buffer de 500 elementos.
    // Verificamos que sea lo mismo que lo enviado.
    //
    wLength = sizeof(buffer);
    wIndex = USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ;
    wValue = 0;
    rc = usb_control_msg( usbdev, CTRL_IN, USBRQ_HID_GET_REPORT, wValue, wIndex, buffer, wLength, 1000 );
    if ( rc <= 0){
        fprintf(stderr, "failed to transfer a few bytes. Error: %d\n", rc);
        usb_close(usbdev);
        return 0;
    }
    printf("Recibido buffer de %d elementos.\n", rc);
    for (i = 0 ; i < rc ; i++)
        {
        if ( buffer[i] != Buffer3[i] ){

            printf("ERROR realizando USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ - no se recibio el mismo valor enviado.\n");
            printf("\telemento Nº%d  --- %d != %d.\n",i, buffer[i] ,Buffer3[i]);
            break;
            }
        }
    printf("Terminado USB_HID_CUSTOM_RQ___BUFFER_DEUNAVEZ.\n");
printf("----\n");
    //=======================================================================
    //=======================================================================

    //
    // Finalizado el programa, liberamos la memoria.
    //
    usb_close(usbdev);
    return 0;
}


usb_dev_handle* usb_find_and_open(int vendorID, char *vendor, int productID, char *product, char *serial)
{
    usb_dev_handle *handle;
    struct usb_bus      *bus;
    struct usb_device   *dev;
    char buffer[60];
    int len=0;

    //
    // Lo primero es obtener una lista de los dispositivos usb conectados.
    // Eso incluye los HUBs y todo el arbol de dispositivos.
    // LibUSB-0.1 nos provee de dos elementos iteradores,
    //    los buses y los dispositivos.
    //

    usb_find_busses();
    usb_find_devices();
    for(bus = usb_get_busses(); bus; bus = bus->next){
        for(dev = bus->devices; dev; dev = dev->next){

        //
        // Para obtener los parámetros del dispositivo debemos pedir su descriptor
        //   de dispositivo en dev->descriptor. Los detalles de sus miembros se pueden leer del mismo header
        //   usb.h y debemos saber que sus nombres estan definidos por el estandard y no por el API.
        //

        //
        // Como primer paso a identificar nuestro dispositivo buscamos cada uno
        //    que tenga el par VID/PID del nuestro. Pero puede haber mas de un
        //    dispositivo conectado con dicho par.
        //
        if ( ! (vendorID==0
             || dev->descriptor.idVendor == vendorID) )
            continue;

        if ( ! (productID == 0
             || dev->descriptor.idProduct == productID) )
            continue;

        printf("--> VID=%x, PID=%x\n",vendorID,productID);

        //
        // Luego necesitaremos entrar en detalles, seleccionar el dispositivo correcto
        //    para ello chequeamos el nombre este pidiendo por los descriptor de String.
        //    En tales, se puede encontrar el nombre textual del manufacturador, del dispositivo
        //    Y mas importante, del numero de serie.
        //
        handle = usb_open(dev);
        if (!handle){
            fprintf(stderr, "failed to open device\n");
            continue;
            }


        len = buffer[0] = 0;
        if(dev->descriptor.iManufacturer > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iManufacturer, buffer, sizeof(buffer));
            if(len < 0)
                fprintf(stderr, "cannot query manufacturer for VID=0x%04x PID=0x%04x\n", vendorID, productID);
            else if( strcmp(buffer, vendor) !=0 )
                {   usb_close(handle);
                    continue;
                }
        }
        else { usb_close(handle);
                continue;
        }

        printf("--> vendor=%s\n",buffer);

        len = buffer[0] = 0;
        if(dev->descriptor.iProduct > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iProduct, buffer, sizeof(buffer));
            if(len < 0)
                fprintf(stderr, "cannot query product for VID=0x%04x PID=0x%04x\n", vendorID, productID);
            else if( strcmp(buffer, product) !=0 )
                {   usb_close(handle);
                    continue;
                }
        }
        else { usb_close(handle);
                continue;
        }
        printf("--> product=%s\n",buffer);


        len = buffer[0] = 0;
        if(dev->descriptor.iSerialNumber > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iSerialNumber, buffer, sizeof(buffer));
            if(len < 0)
                fprintf(stderr, "cannot query SerialNumber for VID=0x%04x PID=0x%04x\n", vendorID, productID);
            else if( strcmp(buffer, serial) !=0 )
                {   usb_close(handle);
                    continue;
                }
        printf("--> SerialNumber=%s\n",buffer);
        }
        else { usb_close(handle);
                continue;
        }

        //
        // Finalmente, para este punto, la variable handle ya contiene el dispositivo,
        //    y es la variable que debemos retornar.
        //
        return handle;
    }}

    //
    // Si el dispositivo no se ha encontrado, retornamos NULL.
    //
    return NULL;
}

