/*
** libusb API:
**    http://libusb.sourceforge.net/api-1.0/
**    http://libusb.sourceforge.net/api-1.0/group__syncio.html
**    http://libusb.sourceforge.net/api-1.0/group__dev.html#g7df95821d20d27b5597f1d783749d6a4
**
*/

#include <stdio.h>
#include <string.h>
#include "../Firmware/usbdrv/usbconfig.h"

#include "../Firmware/defs.h" // Definiciones compartidas

#include <usb.h>

#define CTRL_IN  (USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN)
#define CTRL_OUT (USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT)


usb_dev_handle* usb_find_and_open(int vendorID, char *vendor, int productID, char *product);

int main()
{
    const unsigned char rawVid[2] = {USB_CFG_VENDOR_ID};
    const unsigned char rawPid[2] = {USB_CFG_DEVICE_ID};
    char vendor[] = {USB_CFG_VENDOR_NAME, 0};
    char product[] = {USB_CFG_DEVICE_NAME, 0};
    int vendorID, productID;
    usb_dev_handle* usbdev;
    char buffer[8+1]={0};
    int rc, x;

    vendorID = rawVid[1] * 256 + rawVid[0];
    productID = rawPid[1] * 256 + rawPid[0];

    printf("Buscando: VID=%x, PID=%x, vendor=%s, product=%s\n",vendorID,productID,vendor,product);

	usb_init();

    usbdev = usb_find_and_open(vendorID, vendor, productID, product);
    if (usbdev == NULL)
    {
	    fprintf(stderr, "failed to find device\n");
        return 0;
    }

    printf("Encontrado.\n");

    rc = usb_control_msg( usbdev, CTRL_IN, RQ_CUSTOM___ECHO, 0x3106, 0x2009, buffer, 8, 0 );
    if ( rc < 0){
	    fprintf(stderr, "failed to transfer a few bytes. Error: %d \n",rc);
        usb_close(usbdev);
        return 0;
    }
    if ( !(rc==4 && buffer[0] == 0x06 && buffer[1] == 0x31 && buffer[2] == 0x09 && buffer[3] == 0x20) ){
	    fprintf(stderr, "failed to transfer safelly a few bytes\n");
        usb_close(usbdev);
        return 0;
    }

    printf("Realizado RQ_CUSTOM_ECHO correctamente. Transferido 4 bytes: 0x%08x\n", 0x31062009);

    rc = usb_control_msg( usbdev, CTRL_IN, RQ_CUSTOM___GET_LEDS_STATUS, 0, 0, buffer, 8, 0 );
    if ( rc < 0){
	    fprintf(stderr, "failed to transfer a few bytes\n");
        usb_close(usbdev);
        return 0;
    }
    if ( rc!=1 ){
	    fprintf(stderr, "failed to transfer safelly a few bytes\n");
        usb_close(usbdev);
        return 0;
    }

    printf("Realizado RQ_CUSTOM_GET_LEDS_STATUS correctamente. 5 incrementos. Estado: 0x%02x.\n", buffer[0]);

    for (x = 0 ; x < 5 ; x++){
            buffer[0]++;

            rc = usb_control_msg( usbdev, CTRL_IN, RQ_CUSTOM___SET_LEDS_STATUS, buffer[0], 0, 0, 0, 0 );
            if ( rc < 0){
                fprintf(stderr, "failed to transfer a few bytes\n");
                usb_close(usbdev);
                return 0;
            }
            if ( rc!=0 ){
                fprintf(stderr, "failed to transfer safelly a few bytes  %d\n",rc);
                usb_close(usbdev);
                return 0;
            }
    }

    printf("Realizado 5 RQ_CUSTOM_SET_LEDS_STATUS correctamente.\n");

    usb_close(usbdev);
    return 0;
}


usb_dev_handle* usb_find_and_open(int vendorID, char *vendor, int productID, char *product)
{
    usb_dev_handle *handle;
    struct usb_bus      *bus;
    struct usb_device   *dev;
    char buffer[60];
    int len=0;

    usb_find_busses();
    usb_find_devices();
    for(bus = usb_get_busses(); bus; bus = bus->next){
        for(dev = bus->devices; dev; dev = dev->next){

        if ( ! (vendorID==0
             || dev->descriptor.idVendor == vendorID) )
            continue;

        if ( ! (productID == 0
             || dev->descriptor.idProduct == productID) )
            continue;

        printf("--> VID=%x, PID=%x\n",vendorID,productID);

        handle = usb_open(dev);
        if (!handle){
            fprintf(stderr, "failed to open device\n");
            continue;
            }


        len = buffer[0] = 0;
        if(dev->descriptor.iManufacturer > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iManufacturer, buffer, sizeof(buffer));
            if(len < 0)
                fprintf(stderr, "cannot query manufacturer for VID=0x%04x PID=0x%04x\n", vendorID, productID);
            else if( strcmp(buffer, vendor) !=0 )
                {   usb_close(handle);
                    continue;
                }
        }
        else { usb_close(handle);
                continue;
        }

        printf("--> vendor=%s\n",buffer);

        len = buffer[0] = 0;
        if(dev->descriptor.iProduct > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iProduct, buffer, sizeof(buffer));
            if(len < 0)
                fprintf(stderr, "cannot query product for VID=0x%04x PID=0x%04x\n", vendorID, productID);
            else if( strcmp(buffer, product) !=0 )
                {   usb_close(handle);
                    continue;
                }
        }
        else { usb_close(handle);
                continue;
        }

        printf("--> product=%s\n",buffer);

        return handle;
    }}

    return NULL;
}

