/*
** libusb API:
**    http://libusb.sourceforge.net/api-1.0/
**    http://libusb.sourceforge.net/api-1.0/group__syncio.html
**    http://libusb.sourceforge.net/api-1.0/group__dev.html#g7df95821d20d27b5597f1d783749d6a4
**
*/

#include <stdio.h>
#include <stdlib.h>

#include <usb.h>

int main()
{
	usb_init();
    // discover devices
    usb_dev_handle *handle;
    struct usb_bus      *bus;
    struct usb_device   *dev;

    char buffer[250];
    int len=0;


    //
    // Para buscar el dispositivo entre todos los conectados debemos circular
    //    a través de dos iteradores que provee la librería.
    //
    usb_find_busses();
    usb_find_devices();
    for(bus = usb_get_busses(); bus; bus = bus->next){
    for(dev = bus->devices; dev; dev = dev->next){

        handle = usb_open(dev);

        //
        // El descriptor es miembro del dispositivo.
        //
        printf("idVendor(VID): %x \t",dev->descriptor.idVendor);
        printf("idProduct(PID): %x \n",dev->descriptor.idProduct);


        //
        // Obtener un string descriptor es muy sencillo llamando a la funcion:
        //    int usb_get_string_simple(usb_dev_handle *dev, int index, char *buf, size_t buflen);
        //
        //printf("iManufacturer: %x \t",desc.iManufacturer);
        if(dev->descriptor.iManufacturer > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iManufacturer, buffer, sizeof(buffer));
            buffer[len]='\0';
            printf("Manufacturer: %s\n", buffer);
        }

        //printf("iProduct: %x \t",desc.iProduct);
        if(dev->descriptor.iProduct > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iProduct, buffer, sizeof(buffer));
            buffer[len]='\0';
            printf("Product: %s\n", buffer);
        }

        //printf("iSerialNumber: %x \t",desc.iSerialNumber);
        if(dev->descriptor.iSerialNumber > 0){
            len = usb_get_string_simple(handle, dev->descriptor.iSerialNumber, buffer, sizeof(buffer));
            buffer[len]='\0';
            printf("SerialNumber: %s\n", buffer);
        }

        printf("bLength: %x \t",dev->descriptor.bLength);
        printf("bDeviceClass: %x \n",dev->descriptor.bDeviceClass);
        //
        // Todas las constantes estan definidas en el archivo usb.h
        //
        switch (dev->descriptor.bDeviceClass){
            case USB_CLASS_PER_INTERFACE:
                printf("DeviceClass: PER_INTERFACE\n");
                break;
            case USB_CLASS_AUDIO:
                printf("DeviceClass: UDI\n");
                break;
            case USB_CLASS_COMM:
                printf("DeviceClass: COMM\n");
                break;
            case USB_CLASS_HID:
                printf("DeviceClass: HID\n");
                break;
            case USB_CLASS_PRINTER:
                printf("DeviceClass: PRINTER\n");
                break;
            //case USB_CLASS_PTP:
            //    printf("DeviceClass: PTP\n");
            //    break;
            case USB_CLASS_MASS_STORAGE:
                printf("DeviceClass: MASS_STORAGE\n");
                break;
            case USB_CLASS_HUB:
                printf("DeviceClass: HUB\n");
                break;
            case USB_CLASS_DATA:
                printf("DeviceClass: DATA\n");
                break;
            case USB_CLASS_VENDOR_SPEC:
                printf("DeviceClass: VENDOR_SPEC\n");
                break;
            default:
                printf("DeviceClass: ¿ERROR?\n");
                break;
        }

        //printf("bDescriptorType: %x \t",dev->descriptor.bDescriptorType);
        //printf("bDeviceSubClass: %x \t",dev->descriptor.bDeviceSubClass);
        //printf("bcdUSB: %x \t",dev->descriptor.bcdUSB);
        //printf("bDeviceProtocol: %x \t",dev->descriptor.bDeviceProtocol);
        //printf("bMaxPacketSize0: %x \t",dev->descriptor.bMaxPacketSize0);
        //printf("bcdDevice: %x \t",dev->descriptor.bcdDevice);
        //printf("bNumConfigurations: %x \t",dev->descriptor.bNumConfigurations);
        printf("\n");

        usb_close(handle);
    }}

    return 0;
}



